@@ -16,8 +16,12 @@
 
 #define _FILE_OFFSET_BITS  64
 #include <config.h>
-#include <gnome.h>
-#include <libgnomevfs/gnome-vfs.h>
+#include <gio/gio.h> 
+#include <glib.h>
+#include <glib/gstdio.h>
+#include <glib/gi18n.h>
+#include <gtk/gtk.h>
+
 #include <fcntl.h>
 #include <signal.h>
 #include <stdlib.h>
@@ -42,6 +46,9 @@ static gint gwget_data_process_information (GwgetData *gwgetdata);
 static void gwget_data_update_statistics_ui(GwgetData *gwgetdata);
 static void convert_time2str(gchar *buffer, guint32 time);
 static void gwget_gnotify_finished(GwgetData *gwgetdata);
+static void gwget_download_finished(GwgetData *gwgetdata);
+static gboolean gwget_ask_download_playlist(GwgetData *gwgetdata);
+static void gwget_download_playlist_items(gchar *filename);
 
 static void
 convert_time2str (gchar *buffer, guint32 time)
@@ -50,13 +57,18 @@ convert_time2str (gchar *buffer, guint32 time)
 	     (gint)((time % 3600) / 60), (gint)(time % 60));
 }
 
+void
+gwget_data_set_state_no_sync (GwgetData *gwgetdata, DlState state) 
+{
+	gwgetdata->state=state;
+	gwget_data_update_statistics_ui(gwgetdata);
+}
 
 
 void
 gwget_data_set_state (GwgetData *gwgetdata, DlState state) 
 {
-	gwgetdata->state=state;
-	gwget_data_update_statistics_ui(gwgetdata);
+	gwget_data_set_state_no_sync(gwgetdata, state);
 	gwget_remember_downloads();
 }
 
@@ -213,7 +225,7 @@ gwget_data_update_statistics (GwgetData *gwgetdata)
 		
 	gwget_data_update_statistics_ui(gwgetdata);
 	if (gwgetdata == gwget_data_get_selected() || count_download_in_progress() == 1 ) {
-		gtk_window_set_title(GTK_WINDOW(glade_xml_get_widget(xml, "main_window")), title);
+		gtk_window_set_title(GTK_WINDOW(GTK_WIDGET (gtk_builder_get_object(builder, "main_window"))), title);
 	}
 
 
@@ -223,8 +235,6 @@ gwget_data_update_statistics (GwgetData *gwgetdata)
 static void
 gwget_data_update_statistics_ui(GwgetData *gwgetdata)
 {
-	gchar *tooltip_message;
-	
 	switch(gwgetdata->state) {
 		case DL_WAITING: gwgetdata->state_str = g_strdup(_("Waiting"));
 				 break;
@@ -241,13 +251,6 @@ gwget_data_update_statistics_ui(GwgetData *gwgetdata)
 		case DL_COMPLETED: gwgetdata->state_str=g_strdup(_("Completed"));
 				   gtk_list_store_set(GTK_LIST_STORE(model),&gwgetdata->file_list,
 							      PERCENTAGE_COLUMN,100,-1);
-				   if (gwget_pref.docked) {
-					   tooltip_message=g_strdup_printf(_("Download Completed\n(%s)"), 
-							   gwgetdata->url);
-					   gtk_tooltips_set_tip (GTK_TOOLTIPS(tray_tooltip), GTK_WIDGET(tray_icon), tooltip_message,NULL);
-					   set_icon_idle();
-				   }
-					   
 				   break;
 		case DL_ERROR: 	gwgetdata->state_str = g_strdup_printf("%s: %s",_("Error"),gwgetdata->error_msg);
 				break;
@@ -289,7 +292,7 @@ gwget_data_process_information (GwgetData *gwgetdata)
 		else if (WIFEXITED (status)) {
             		if (WEXITSTATUS (status) == 0) {
                 		gwget_data_set_state (gwgetdata, DL_COMPLETED);
-                     		gwget_gnotify_finished(gwgetdata);
+                     		gwget_download_finished(gwgetdata);
 				start_first_waiting_download();
 				if (gwget_pref.open_after_dl) {
 					gwget_data_exec(gwgetdata);
@@ -338,12 +341,12 @@ gwget_data_start_download(GwgetData *gwgetdata)
 	number=count_download_in_progress();
 
 	if (gwget_pref.limit_simultaneousdownloads==TRUE && number>=gwget_pref.max_simultaneousdownloads) {
-		gwget_data_set_state(gwgetdata,DL_WAITING);
+		gwget_data_set_state_no_sync(gwgetdata,DL_WAITING);
 		return;
 	}
 
 	/* Put the not connected state before */
-	gwget_data_set_state(gwgetdata,DL_NOT_CONNECTED);
+	gwget_data_set_state_no_sync(gwgetdata,DL_NOT_CONNECTED);
 	gwget_data_update_statistics_ui(gwgetdata);
 	
 	/* First check if we are not starting an already started download */
@@ -443,8 +446,8 @@ gwget_data_start_download(GwgetData *gwgetdata)
 					user_password=g_strdup_printf("%s:%s@",gwget_pref.proxy_user,gwget_pref.proxy_password);
 				}
 
-				setenv("http_proxy", g_strconcat("http://",g_strdup_printf(user_password),g_strdup_printf("%s",gwget_pref.http_proxy),":",g_strdup_printf("%d",gwget_pref.http_proxy_port),NULL), 1);
-				setenv("ftp_proxy", g_strconcat("http://",g_strdup_printf(user_password),g_strdup_printf("%s",gwget_pref.http_proxy),":",g_strdup_printf("%d",gwget_pref.http_proxy_port),NULL), 1);
+				setenv("http_proxy", g_strconcat("http://", g_strdup_printf("%s", user_password),g_strdup_printf("%s",gwget_pref.http_proxy),":",g_strdup_printf("%d",gwget_pref.http_proxy_port),NULL), 1);
+				setenv("ftp_proxy", g_strconcat("http://", g_strdup_printf("%s", user_password),g_strdup_printf("%s",gwget_pref.http_proxy),":",g_strdup_printf("%d",gwget_pref.http_proxy_port),NULL), 1);
 		 	        argv[arg]="-Yon";
 				arg++;
 			}
@@ -456,8 +459,8 @@ gwget_data_start_download(GwgetData *gwgetdata)
 					user_password=g_strdup_printf("%s:%s@",gwget_pref.gnome_proxy_user,gwget_pref.gnome_proxy_password);
 				}
 
-				setenv("http_proxy",g_strconcat("http://",g_strdup_printf(user_password),g_strdup_printf("%s",gwget_pref.http_proxy),":",g_strdup_printf("%d",gwget_pref.http_proxy_port),NULL),1);
-				setenv("ftp_proxy",g_strconcat("http://",g_strdup_printf(user_password),g_strdup_printf("%s",gwget_pref.http_proxy),":",g_strdup_printf("%d",gwget_pref.http_proxy_port),NULL),1);
+				setenv("http_proxy",g_strconcat("http://", g_strdup_printf("%s", user_password), g_strdup_printf("%s", gwget_pref.gnome_http_proxy),":", g_strdup_printf("%d", gwget_pref.gnome_http_proxy_port),NULL),1);
+				setenv("ftp_proxy",g_strconcat("http://", g_strdup_printf("%s", user_password), g_strdup_printf("%s", gwget_pref.gnome_http_proxy),":", g_strdup_printf("%d", gwget_pref.gnome_http_proxy_port),NULL),1);
 		 	        argv[arg]="-Yon";
 				arg++;
 			}
@@ -506,13 +509,22 @@ gwget_data_set_total_size (GwgetData *gwgetdata, guint64 total_size)
 	gwgetdata->total_size = total_size;
 }
 
+GwgetData *
+gwget_data_new (gchar *url)
+{
+
+	return gwget_data_create (url, gwget_pref.download_dir);
+
+}
 
 GwgetData * 
 gwget_data_create(gchar *url, gchar *dir)
 {
 	GwgetData *gwgetdata;
-	GnomeVFSURI *localfile_uri;
+	GFile *file;
+	GFileInfo *info;
 	gint length;
+	GError    *err = NULL;
 
 	g_return_val_if_fail(url != NULL, NULL);
 	g_return_val_if_fail(dir != NULL, NULL);
@@ -540,23 +552,21 @@ gwget_data_create(gchar *url, gchar *dir)
 	
 	gwget_data_set_filename(gwgetdata,gwgetdata->filename);
 	
-	localfile_uri = gnome_vfs_uri_new (gwgetdata->local_filename);
-	if (gnome_vfs_uri_exists (localfile_uri) ) {
-		GnomeVFSFileInfo *info;
-		info = gnome_vfs_file_info_new();
-		gnome_vfs_get_file_info (gwgetdata->local_filename, 
-				         info, 
-					 GNOME_VFS_FILE_INFO_DEFAULT);
-		gwgetdata->cur_size = (guint64)info->size;
+	file = g_file_new_for_path (gwgetdata->local_filename);
+	info = g_file_query_info (file, G_FILE_ATTRIBUTE_STANDARD_SIZE, 0, NULL, &err);
+	
+	if (err==NULL) {
+		gwgetdata->cur_size = g_file_info_get_size (info);
 	} else {
 		gwgetdata->cur_size = 0;
+		g_error_free (err);
 	}
 	
+	
 	gwgetdata->line = NULL;
 	gwgetdata->session_start_time = 0;
 	gwgetdata->session_start_size = 0;
 	gwgetdata->session_elapsed = 0;
-	/* gwgetdata->cur_size = 0; */
 	gwgetdata->state = DL_NOT_STARTED;
 	gwgetdata->total_size = 0;
 	gwgetdata->total_time = 0;
@@ -576,7 +586,6 @@ gwget_data_set_filename(GwgetData* gwgetdata,gchar *str)
 	gwgetdata->local_filename = g_strconcat (gwgetdata->dir, 
 			                         str,
 						 NULL);
-	gwget_remember_downloads();
 }
 
 /* Return the gwgetdata that is selected in the treeview */
@@ -589,7 +598,7 @@ GwgetData* gwget_data_get_selected(void)
 	GwgetData *gwgetdata;
 	gchar *url;
 	
-	treev=glade_xml_get_widget(xml,"treeview1");
+	treev=GTK_WIDGET (gtk_builder_get_object(builder,"treeview1"));
 	select=gtk_tree_view_get_selection(GTK_TREE_VIEW(treev));
 	
 	if (gtk_tree_selection_get_selected (select, &model, &iter)) 
@@ -670,8 +679,6 @@ gwget_data_set_filename_from_url(GwgetData *gwgetdata,gchar *url)
 	} else {
 		gwgetdata->filename = g_strdup(filename);
 	}
-
-	gwget_remember_downloads();
 }
 
 /* Add a gwgetdata to the main window */
@@ -694,44 +701,47 @@ gwget_data_add_download(GwgetData *gwgetdata)
 	if (!strcmp(gwgetdata->filename,"") || !strcmp(gwgetdata->filename,gwgetdata->url) ||
 		!strncmp(reverse_filename,"lmth",4) || !strncmp(reverse_filename,"mth",3) || 
 		!strncmp(reverse_filename,"php",3)  || !strncmp(reverse_filename,"asp",3)) {
-		recursive_window=glade_xml_get_widget(xml,"dialog2");
+		recursive_window=GTK_WIDGET (gtk_builder_get_object(builder,"dialog2"));
 		response=gtk_dialog_run(GTK_DIALOG(recursive_window));
 		gtk_widget_hide(GTK_WIDGET(recursive_window));
 		if (response==GTK_RESPONSE_OK) {
-			radio=glade_xml_get_widget(xml,"radio_index");
+			radio=GTK_WIDGET (gtk_builder_get_object(builder,"radio_index"));
 			if (gtk_toggle_button_get_active(GTK_TOGGLE_BUTTON(radio))) {
 				gwgetdata->recursive=FALSE;
 			}
-			radio=glade_xml_get_widget(xml,"radio_multimedia");
+			radio=GTK_WIDGET (gtk_builder_get_object(builder,"radio_multimedia"));
 			if (gtk_toggle_button_get_active(GTK_TOGGLE_BUTTON(radio))) {
 				gwgetdata->recursive=TRUE;
 				gwgetdata->multimedia=TRUE;
 			}
-			radio=glade_xml_get_widget(xml,"radio_mirror");
+			radio=GTK_WIDGET (gtk_builder_get_object(builder,"radio_mirror"));
 			if (gtk_toggle_button_get_active(GTK_TOGGLE_BUTTON(radio))) {
 				gwgetdata->recursive=TRUE;
 				gwgetdata->mirror=TRUE;
 			}
-			radio=glade_xml_get_widget(xml,"radio_recursive");
+			radio=GTK_WIDGET (gtk_builder_get_object(builder,"radio_recursive"));
 			if (gtk_toggle_button_get_active(GTK_TOGGLE_BUTTON(radio))) {
 				gwgetdata->recursive=TRUE;
 			}
 		}
 	}
 	downloads = g_list_append(downloads,gwgetdata);
-	gwget_data_set_state(gwgetdata,DL_NOT_CONNECTED);
 	new_download(gwgetdata);
+	gwget_data_set_state(gwgetdata,DL_NOT_CONNECTED);
 }
 
 void
 gwget_data_exec (GwgetData *gwgetdata)
 {
-	gchar *uri;
+	gchar *url;
+	GFile *file;
 	GError *err = NULL; 
 
-	uri = gnome_vfs_make_uri_from_input_with_dirs (gwgetdata->local_filename,
-							GNOME_VFS_MAKE_URI_DIR_CURRENT);
-	if (!gnome_url_show (uri, &err)) {
+	file = g_file_new_for_commandline_arg (gwgetdata->local_filename);
+	url = g_file_get_uri (file);
+	g_object_unref (file);
+	
+	if (!gtk_show_uri (NULL, url, GDK_CURRENT_TIME, &err)) {
 		run_dialog_error(_("Error opening file"), _("Couldn't open the file"));
 	}
 }
@@ -770,50 +780,111 @@ gwget_gnotify_finished(GwgetData *gwgetdata) {
       close(sock);
    }
 
-   gwget_tray_notify (_("Download Complete"), gwgetdata->filename, gwgetdata->icon_name);
    g_free(app_msg);
    g_free(icon_msg);
    g_free(fn_msg);
-   g_free(gwgetdata->icon_name);
+}
+
+
+static void
+gwget_download_finished (GwgetData *gwgetdata)
+{
+	gwget_gnotify_finished(gwgetdata);
+	gwget_tray_notify (_("Download Complete"), gwgetdata->filename, gwgetdata->icon_name);
+	gtk_widget_set_sensitive(GTK_WIDGET (gtk_builder_get_object(builder, "clear_button")), TRUE);
+
+	if (gwget_ask_download_playlist(gwgetdata))
+		gwget_download_playlist_items(gwgetdata->local_filename);
+}
+
+static gboolean
+gwget_ask_download_playlist(GwgetData *gwgetdata)
+{
+	gchar *msg;
+	gint response;
+
+	if (g_str_has_suffix(gwgetdata->filename, ".m3u")
+	    	|| g_str_has_suffix(gwgetdata->filename, ".M3U")) {   
+		msg = g_strdup_printf(
+			_("The file %s is an MP3 playlist.\nDownload the files that it contains?"),
+			gwgetdata->filename);
+		response = run_dialog(_("Download files in MP3 playlist?"),
+							  _(msg), _("No"), _("Yes"));
+		g_free(msg);
+		if (response == GTK_RESPONSE_OK)
+			return TRUE;
+	}
+	
+	return FALSE;
+}
+
+static void
+gwget_download_playlist_items(gchar *filename)
+{
+	FILE *f;
+	gchar line[1024];
+	GwgetData *gwgetdata;
+	
+	f = g_fopen(filename, "r");
+	if (f!=NULL) {
+		while (fgets(line, 1024, f)!=NULL) {
+			if (check_url("http://", line) || check_url("ftp://", line)) {
+				gwgetdata = gwget_data_new (g_strstrip(line));
+				gwget_data_add_download(gwgetdata);
+				gwget_data_start_download(gwgetdata);
+			}
+		}
+		fclose(f);
+	}
 }
 
 void
 gwget_data_set_menus (GwgetData *gwgetdata)
 {
 
-
 	if (gwgetdata!=NULL) {
 		if (gwget_data_run(gwgetdata)) {
-			/* Menu items */
-			gtk_widget_set_sensitive(glade_xml_get_widget(xml,"pause_menuitem"),TRUE);
-			gtk_widget_set_sensitive(glade_xml_get_widget(xml,"continue_menuitem"),FALSE);
-			gtk_widget_set_sensitive(glade_xml_get_widget(xml,"cancel_menuitem"),TRUE);
-			/* popup */
-			gtk_widget_set_sensitive(glade_xml_get_widget(xml,"pause_download"),TRUE);
-			gtk_widget_set_sensitive(glade_xml_get_widget(xml,"continue_download"),FALSE);
-			gtk_widget_set_sensitive(glade_xml_get_widget(xml,"cancel_download"),TRUE);
+			gtk_widget_set_sensitive(GTK_WIDGET (gtk_builder_get_object(builder,"pause_menuitem")),TRUE);
+			gtk_widget_set_sensitive(GTK_WIDGET (gtk_builder_get_object(builder,"continue_menuitem")),FALSE);
+			gtk_widget_set_sensitive(GTK_WIDGET (gtk_builder_get_object(builder,"cancel_menuitem")),TRUE);
 		} else {
 			if (gwgetdata->state==DL_COMPLETED) {
-				gtk_widget_set_sensitive(glade_xml_get_widget(xml,"continue_menuitem"),FALSE);
-				gtk_widget_set_sensitive(glade_xml_get_widget(xml,"pause_menuitem"),FALSE);
-				gtk_widget_set_sensitive(glade_xml_get_widget(xml,"cancel_menuitem"),FALSE);
-				gtk_widget_set_sensitive(glade_xml_get_widget(xml,"continue_download"),FALSE);
-				gtk_widget_set_sensitive(glade_xml_get_widget(xml,"pause_download"),FALSE);
-				gtk_widget_set_sensitive(glade_xml_get_widget(xml,"cancel_download"),FALSE);
+				gtk_widget_set_sensitive(GTK_WIDGET (gtk_builder_get_object(builder,"continue_menuitem")),FALSE);
+				gtk_widget_set_sensitive(GTK_WIDGET (gtk_builder_get_object(builder,"pause_menuitem")),FALSE);
+				gtk_widget_set_sensitive(GTK_WIDGET (gtk_builder_get_object(builder,"cancel_menuitem")),FALSE);
 			} else {
-				gtk_widget_set_sensitive(glade_xml_get_widget(xml,"continue_menuitem"),TRUE);
-				gtk_widget_set_sensitive(glade_xml_get_widget(xml,"pause_menuitem"),FALSE);
-				gtk_widget_set_sensitive(glade_xml_get_widget(xml,"continue_download"),TRUE);
-				gtk_widget_set_sensitive(glade_xml_get_widget(xml,"pause_download"),FALSE);
+				gtk_widget_set_sensitive(GTK_WIDGET (gtk_builder_get_object(builder,"continue_menuitem")),TRUE);
+				gtk_widget_set_sensitive(GTK_WIDGET (gtk_builder_get_object(builder,"pause_menuitem")),FALSE);
 			}
 		}
 	} else {
-			gtk_widget_set_sensitive(glade_xml_get_widget(xml,"continue_menuitem"),FALSE);
-			gtk_widget_set_sensitive(glade_xml_get_widget(xml,"pause_menuitem"),FALSE);
-			gtk_widget_set_sensitive(glade_xml_get_widget(xml,"cancel_menuitem"),FALSE);
-			gtk_widget_set_sensitive(glade_xml_get_widget(xml,"continue_download"),FALSE);
-			gtk_widget_set_sensitive(glade_xml_get_widget(xml,"pause_download"),FALSE);
-			gtk_widget_set_sensitive(glade_xml_get_widget(xml,"cancel_download"),FALSE);
-			gtk_widget_set_sensitive(glade_xml_get_widget(xml,"properties_menuitem"),FALSE);
+		gtk_widget_set_sensitive(GTK_WIDGET (gtk_builder_get_object(builder,"continue_menuitem")),FALSE);
+		gtk_widget_set_sensitive(GTK_WIDGET (gtk_builder_get_object(builder,"pause_menuitem")),FALSE);
+		gtk_widget_set_sensitive(GTK_WIDGET (gtk_builder_get_object(builder,"cancel_menuitem")),FALSE);
 	}
 }
+
+void 
+gwget_data_set_popupmenu (GwgetData *gwgetdata)
+{
+	if (gwgetdata!=NULL) {
+		if (gwget_data_run(gwgetdata)) {
+			gtk_widget_set_sensitive(GTK_WIDGET (gtk_builder_get_object(builder,"pause_download")),TRUE);
+                        gtk_widget_set_sensitive(GTK_WIDGET (gtk_builder_get_object(builder,"continue_download")),FALSE);
+                        gtk_widget_set_sensitive(GTK_WIDGET (gtk_builder_get_object(builder,"cancel_download")),TRUE);
+		} else {
+			if (gwgetdata->state==DL_COMPLETED) {
+                                gtk_widget_set_sensitive(GTK_WIDGET (gtk_builder_get_object(builder,"continue_download")),FALSE);
+                                gtk_widget_set_sensitive(GTK_WIDGET (gtk_builder_get_object(builder,"pause_download")),FALSE);
+                                gtk_widget_set_sensitive(GTK_WIDGET (gtk_builder_get_object(builder,"cancel_download")),FALSE);
+                        } else {
+                                gtk_widget_set_sensitive(GTK_WIDGET (gtk_builder_get_object(builder,"continue_download")),TRUE);
+                                gtk_widget_set_sensitive(GTK_WIDGET (gtk_builder_get_object(builder,"pause_download")),FALSE);
+                        }
+		}
+	} else {
+                gtk_widget_set_sensitive(GTK_WIDGET (gtk_builder_get_object(builder,"continue_download")),FALSE);
+                gtk_widget_set_sensitive(GTK_WIDGET (gtk_builder_get_object(builder,"pause_download")),FALSE);
+                gtk_widget_set_sensitive(GTK_WIDGET (gtk_builder_get_object(builder,"cancel_download")),FALSE);
+        }
+}
